mt4_fun_num_map = {
"CopyLow_001": 1
, "TimeHour": 2
, "StringToInteger": 3
, "AccountEquity": 4
, "GlobalVariableDel": 5
, "IsTradeAllowed": 6
, "FolderDelete": 7
, "MQLInfoInteger": 8
, "SignalInfoGetString": 9
, "OrdersHistoryTotal": 10
, "ObjectsDeleteAll_001": 11
, "CharArrayToString": 12
, "StringToDouble": 13
, "iBarShift": 14
, "ObjectName": 15
, "StringTrimRight": 16
, "ArrayFree": 17
, "WindowFirstVisibleBar": 18
, "StringSubstr": 19
, "iAD": 20
, "SignalBaseTotal": 21
, "ObjectGetShiftByValue": 22
, "StringGetCharacter": 23
, "fmax": 24
, "SetIndexEmptyValue": 25
, "ChartScreenShot": 26
, "StringLen": 27
, "OrderSend": 28
, "OrderDelete": 29
, "OrderProfit": 30
, "StringSetCharacter": 31
, "WindowRedraw": 32
, "AccountName": 33
, "CryptDecode": 34
, "FolderClean": 35
, "IsTesting": 36
, "ChartFirst": 37
, "OrderModify": 38
, "HideTestIndicators": 39
, "TesterStatistics": 40
, "ExpertRemove": 41
, "ceil": 42
, "SignalBaseGetInteger": 43
, "AccountCurrency": 44
, "ChartIndicatorsTotal": 45
, "StringAdd": 46
, "ObjectGetDouble": 47
, "DayOfYear": 48
, "iMA": 49
, "MathRand": 50
, "iAO": 51
, "iSAR": 52
, "cos": 53
, "iOpen": 54
, "Symbol": 55
, "WindowExpertName": 56
, "Year": 57
, "GlobalVariableGet": 58
, "ObjectSetString_002": 59
, "iLow": 60
, "DebugBreak": 61
, "WebRequest": 62
, "StringInit": 63
, "GetLastError": 64
, "OnInit": 65
, "FolderCreate": 66
, "WindowOnDropped": 67
, "GlobalVariableTime": 68
, "FileMove": 69
, "FileOpen": 70
, "FileWriteString": 71
, "iTime": 72
, "CheckPointer": 73
, "OrderSwap": 74
, "ChartSaveTemplate": 75
, "TextOut": 76
, "Sleep": 77
, "MathCos": 78
, "sin": 79
, "ChartID": 80
, "CopyTickVolume_001": 81
, "StringCompare": 82
, "ObjectSetFiboDescription": 83
, "TimeMinute": 84
, "FileOpenHistory": 85
, "ZeroMemory": 86
, "iCustom": 87
, "srand": 88
, "SendFTP": 89
, "SignalInfoGetDouble": 90
, "AccountNumber": 91
, "StringFind": 92
, "IsLibrariesAllowed": 93
, "iEnvelopes": 94
, "StringFormat": 95
, "ArrayCopy": 96
, "iWPR": 97
, "WindowYOnDropped": 98
, "ArrayMinimum": 99
, "OrderSymbol": 100
, "CopyTime_003": 101
, "StringFill": 102
, "DayOfWeek": 103
, "AccountStopoutMode": 104
, "Day": 105
, "GetTickCount": 106
, "ArrayCompare": 107
, "ObjectGetTimeByValue": 108
, "SignalInfoSetDouble": 109
, "TimeDayOfYear": 110
, "GlobalVariableTemp": 111
, "FileFlush": 112
, "StringSplit": 113
, "rand": 114
, "IndicatorSetString": 115
, "MathArctan": 116
, "GetPointer": 117
, "AccountProfit": 118
, "Hour": 119
, "IndicatorSetDouble": 120
, "MathMax": 121
, "iRVI": 122
, "Digits": 123
, "FileDelete": 124
, "MathCeil": 125
, "OnChartEvent": 126
, "CopyRates_003": 127
, "StringReplace": 128
, "iAlligator": 129
, "FileWriteLong": 130
, "WindowPriceMax": 131
, "iRSIOnArray": 132
, "iHighest": 133
, "round": 134
, "CopyRates_002": 135
, "FileTell": 136
, "Bars_002": 137
, "TimeCurrent": 138
, "TerminalInfoInteger": 139
, "CopyTickVolume_003": 140
, "MathArccos": 141
, "TextGetSize": 142
, "AccountMargin": 143
, "MarketInfo": 144
, "WindowPriceOnDropped": 145
, "ObjectsDeleteAll_003": 146
, "OnDeinit": 147
, "ChartGetInteger": 148
, "GlobalVariablesFlush": 149
, "ObjectFind_002": 150
, "ResourceCreate": 151
, "GlobalVariableName": 152
, "FileReadDatetime": 153
, "ObjectGetValueByShift": 154
, "SetIndexLabel": 155
, "Minute": 156
, "WindowXOnDropped": 157
, "SetIndexArrow": 158
, "FileReadLong": 159
, "fmin": 160
, "IsExpertEnabled": 161
, "Point": 162
, "ObjectGetString": 163
, "EnumToString": 164
, "MathIsValidNumber": 165
, "ColorToARGB": 166
, "FileFindClose": 167
, "StringToColor": 168
, "StringTrimLeft": 169
, "iIchimoku": 170
, "TerminalCompany": 171
, "NormalizeDouble": 172
, "ChartWindowOnDropped": 173
, "GlobalVariablesDeleteAll": 174
, "ChartWindowFind": 175
, "MathExp": 176
, "StringToShortArray": 177
, "OrdersTotal": 178
, "TimeDayOfWeek": 179
, "IsDemo": 180
, "MQLInfoString": 181
, "CopyLow_002": 182
, "SymbolInfoDouble": 183
, "EventSetMillisecondTimer": 184
, "AccountBalance": 185
, "atan": 186
, "WindowBarsPerChart": 187
, "SetIndexBuffer": 188
, "MathLog10": 189
, "ObjectDelete_002": 190
, "AccountInfoInteger": 191
, "OrderOpenPrice": 192
, "GlobalVariableSet": 193
, "CopyHigh_003": 194
, "StringBufferLen": 195
, "MathRound": 196
, "FileFindFirst": 197
, "WindowsTotal": 198
, "FileIsExist": 199
, "ArrayMaximum": 200
, "iAC": 201
, "StrToDouble": 202
, "ChartTimeOnDropped": 203
, "ArrayGetAsSeries": 204
, "ArrayIsDynamic": 205
, "MathArcsin": 206
, "SymbolInfoTick": 207
, "TimeDaylightSavings": 208
, "IntegerToString": 209
, "AccountFreeMargin": 210
, "SymbolInfoSessionQuote": 211
, "fabs": 212
, "EventKillTimer": 213
, "ChartSetDouble": 214
, "iBullsPower": 215
, "fmod": 216
, "OrderPrint": 217
, "SetIndexShift": 218
, "PrintFormat": 219
, "MathSqrt": 220
, "ShortToString": 221
, "FileIsEnding": 222
, "PeriodSeconds": 223
, "IsStopped": 224
, "OnTimer": 225
, "pow": 226
, "FileWrite": 227
, "ResourceFree": 228
, "CopyHigh_001": 229
, "iStochastic": 230
, "OrderOpenTime": 231
, "DoubleToString": 232
, "GlobalVariableSetOnCondition": 233
, "ChartTimePriceToXY": 234
, "ArrayDimension": 235
, "ChartIndicatorDelete": 236
, "OrderExpiration": 237
, "SendMail": 238
, "ChartSetSymbolPeriod": 239
, "ArrayCopyRates": 240
, "SeriesInfoInteger": 241
, "MQLSetInteger": 242
, "iATR": 243
, "FileReadArray": 244
, "ObjectGetValueByTime": 245
, "CharToString": 246
, "ObjectDescription": 247
, "FileReadDouble": 248
, "FileWriteStruct": 249
, "FileFindNext": 250
, "OrderCloseTime": 251
, "iMACD": 252
, "iOsMA": 253
, "floor": 254
, "StrToTime": 255
, "RefreshRates": 256
, "iBands": 257
, "TerminalClose": 258
, "SignalInfoGetInteger": 259
, "ResourceSave": 260
, "iLowest": 261
, "CopyLow_003": 262
, "FileWriteFloat": 263
, "ResourceReadImage": 264
, "MathSin": 265
, "ChartPriceOnDropped": 266
, "FileReadBool": 267
, "ShortArrayToString": 268
, "ChartIndicatorName": 269
, "SetIndexDrawBegin": 270
, "ObjectGetInteger": 271
, "SignalUnsubscribe": 272
, "IndicatorBuffers": 273
, "EventChartCustom": 274
, "iBandsOnArray": 275
, "FileCopy": 276
, "OrderClose": 277
, "ChartOpen": 278
, "iMFI": 279
, "ObjectsTotal_001": 280
, "OnTester": 281
, "CopyClose_001": 282
, "FileGetInteger": 283
, "asin": 284
, "FileReadNumber": 285
, "AccountInfoDouble": 286
, "CopyTime_001": 287
, "CopyClose_002": 288
, "SendNotification": 289
, "acos": 290
, "EventSetTimer": 291
, "CopyTickVolume_002": 292
, "IndicatorSetInteger": 293
, "SignalInfoSetInteger": 294
, "MathLog": 295
, "ObjectSetInteger_002": 296
, "CopyRates_001": 297
, "log10": 298
, "ChartApplyTemplate": 299
, "ObjectCreate_002": 300
, "MathMin": 301
, "OrderType": 302
, "ObjectType": 303
, "ArrayResize": 304
, "MathTan": 305
, "CopyOpen_003": 306
, "SymbolName": 307
, "CopyOpen_001": 308
, "MathSrand": 309
, "FileSize": 310
, "OrderMagicNumber": 311
, "ChartXYToTimePrice": 312
, "MathFloor": 313
, "MessageBox": 314
, "FileWriteArray": 315
, "GlobalVariablesTotal": 316
, "iGator": 317
, "SetLevelStyle": 318
, "iStdDev": 319
, "iADX": 320
, "IndicatorCounted": 321
, "ArraySize": 322
, "FileSeek": 323
, "SignalBaseGetDouble": 324
, "iRSI": 325
, "iCCIOnArray": 326
, "iMomentumOnArray": 327
, "OrderTakeProfit": 328
, "WindowScreenShot": 329
, "ObjectSetString_001": 330
, "ChartNext": 331
, "ChartGetString": 332
, "ObjectGetFiboDescription": 333
, "exp": 334
, "ObjectSetText": 335
, "iOBV": 336
, "CopyHigh_002": 337
, "ObjectsTotal_002": 338
, "OrderLots": 339
, "iDeMarker": 340
, "ArrayRange": 341
, "DoubleToStr": 342
, "tan": 343
, "ChartRedraw": 344
, "SignalBaseSelect": 345
, "OnTick": 346
, "UninitializeReason": 347
, "log": 348
, "TextSetFont": 349
, "WindowFind": 350
, "ChartGetDouble": 351
, "SetLevelValue": 352
, "iStdDevOnArray": 353
, "ObjectsDeleteAll_002": 354
, "MathPow": 355
, "Comment": 356
, "CopyOpen_002": 357
, "SymbolsTotal": 358
, "FileReadStruct": 359
, "Month": 360
, "ArrayCopySeries": 361
, "iMAOnArray": 362
, "IsOptimization": 363
, "SymbolInfoString": 364
, "ObjectCreate_001": 365
, "TerminalInfoDouble": 366
, "OrderStopLoss": 367
, "IsConnected": 368
, "AccountFreeMarginCheck": 369
, "FileWriteInteger": 370
, "ObjectSetInteger_001": 371
, "CopyTime_002": 372
, "ArraySort": 373
, "SignalSubscribe": 374
, "AccountCredit": 375
, "OrderSelect": 376
, "FileReadFloat": 377
, "ChartNavigate": 378
, "ResetLastError": 379
, "iForce": 380
, "iBearsPower": 381
, "OrderComment": 382
, "OrderTicket": 383
, "IsVisualMode": 384
, "AccountInfoString": 385
, "SetIndexStyle": 386
, "ChartPeriod": 387
, "iVolume": 388
, "iHigh": 389
, "AccountFreeMarginMode": 390
, "Seconds": 391
, "Alert": 392
, "ArrayIsSeries": 393
, "OrderCloseBy": 394
, "StringGetChar": 395
, "ArrayFill": 396
, "StringToCharArray": 397
, "StringToLower": 398
, "Bars_001": 399
, "IndicatorShortName": 400
, "ChartSymbol": 401
, "MathMod": 402
, "ChartClose": 403
, "ChartSetString": 404
, "ObjectSet": 405
, "iBWMFI": 406
, "ChartSetInteger": 407
, "AccountCompany": 408
, "MathAbs": 409
, "ObjectGet": 410
, "FileIsLineEnding": 411
, "StringConcatenate": 412
, "TimeGMTOffset": 413
, "ObjectSetDouble_001": 414
, "AccountServer": 415
, "ArrayBsearch": 416
, "OrderClosePrice": 417
, "TerminalInfoString": 418
, "ObjectMove": 419
, "TimeToStr": 420
, "ObjectDelete_001": 421
, "FileReadInteger": 422
, "iClose": 423
, "iFractals": 424
, "StringToTime": 425
, "Print": 426
, "GlobalVariableCheck": 427
, "TimeYear": 428
, "ObjectFind_001": 429
, "OrderCommission": 430
, "SymbolSelect": 431
, "IsTradeContextBusy": 432
, "StringToUpper": 433
, "ChartXOnDropped": 434
, "iBars": 435
, "TimeToStruct": 436
, "ArraySetAsSeries": 437
, "TerminalPath": 438
, "AccountStopoutLevel": 439
, "ColorToString": 440
, "sqrt": 441
, "TimeGMT": 442
, "SignalBaseGetString": 443
, "WindowIsVisible": 444
, "WindowTimeOnDropped": 445
, "Period": 446
, "StringSetChar": 447
, "iEnvelopesOnArray": 448
, "TimeMonth": 449
, "TimeLocal": 450
, "CharToStr": 451
, "IndicatorDigits": 452
, "WindowHandle": 453
, "SymbolInfoSessionTrade": 454
, "AccountLeverage": 455
, "FileReadString": 456
, "ObjectSetDouble_002": 457
, "iCCI": 458
, "TimeSeconds": 459
, "ArrayInitialize": 460
, "CryptEncode": 461
, "StructToTime": 462
, "IsDllsAllowed": 463
, "CopyClose_003": 464
, "PlaySound": 465
, "iMomentum": 466
, "TimeDay": 467
, "FileWriteDouble": 468
, "FileClose": 469
, "TerminalName": 470
, "StrToInteger": 471
, "SymbolInfoInteger": 472
, "TimeToString": 473
, "WindowPriceMin": 474
, "ChartYOnDropped": 475
}
